package com.log4j.michael;

import java.io.Writer;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.DatagramPacket;
import java.net.UnknownHostException;
import java.net.SocketException;
import java.io.IOException;
import java.net.URL;
import java.net.MalformedURLException;

import org.apache.log4j.helpers.LogLog;

/**
   SyslogWriter is a wrapper around the java.net.DatagramSocket class
   so that it behaves like a java.io.Writer.

   @since 0.7.3
*/
public class SyslogSunjWriter extends Writer {

  final int SYSLOG_PORT = 514;
  /**
   *  Host string from last constructed SyslogWriter.
   *  @deprecated
   */
  static String syslogHost;
  
  private InetAddress address;
  private final int port;
  private DatagramSocket ds;
  


  public
  SyslogSunjWriter(final String syslogHost,final String syslogPort) {
      SyslogSunjWriter.syslogHost = syslogHost;
    if (syslogHost == null) {
        throw new NullPointerException("syslogHost");
    }
    
    String host = syslogHost;
    
    
    int urlPort = -1;
    if(null!=syslogPort && !"".equals(syslogPort)){
        try{
            urlPort = Integer.parseInt(syslogPort);
        }catch(Exception e){
            urlPort=-1;
        }
    }
    
    if (urlPort == -1) {
        urlPort = SYSLOG_PORT;
    }
    port = urlPort;

    try {      
      this.address = InetAddress.getByName(host);
    }
    catch (UnknownHostException e) {
      LogLog.error("Could not find " + host +
             ". All logging will FAIL.", e);
    }

    try {
      this.ds = new DatagramSocket();
    }
    catch (SocketException e) {
      e.printStackTrace();
      LogLog.error("Could not instantiate DatagramSocket to " + host +
             ". All logging will FAIL.", e);
    }
    
  }


  public
  void write(char[] buf, int off, int len) throws IOException {
    this.write(new String(buf, off, len));
  }
  
  public
  void write(final String string) throws IOException {

    if(this.ds != null && this.address != null) {
        byte[] bytes = string.getBytes();
        //
        //  syslog packets must be less than 1024 bytes
        //
        int bytesLength = bytes.length;
        if (bytesLength >= 1024) {
            bytesLength = 1024;
        }
        DatagramPacket packet = new DatagramPacket(bytes, bytesLength,
                               address, port);
        ds.send(packet);
    }
    
  }

  public
  void flush() {}

  public void close() {
      if (ds != null) {
          ds.close();
      }
  }
}
